/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.datasources.interlib

import groovy.sql.Sql;
import metridoc.plugins.table.core.Datasource;
import metridoc.datasources.interlib.service.BorrowDirectService;

class BdSummaryDatasource implements Datasource{

	def datasource = [:]

	public Object getInputs() {
		return [instance:["url":"","username":"","password":""],
			request: ["fiscalYear": null]];
	}

	public Object profile() {
		def profile = [
					main_report:
					[table_name: 'Borrowing',
						columns: [
							[column_name:'library',
								type_name:"String"],
							[column_name:'req_rec',
								type_name:"float"],
							[column_name:'req_shp',
								type_name:"float"],
							[column_name:'shp_rec',
								type_name:"float"],
							[column_name:'year_items',
								type_name:"float"],
							[column_name:'year_fill_rate',
								type_name:"float"],
							[column_name:'year_last_year',
								type_name:"float"]
						]]
				]
		def columns = profile.main_report.columns
		(1 .. 12).each {
			columns.add([column_name: it+'_items',
						type_name:"float"]);
			columns.add([column_name:it+'_last_year',
						type_name:"float"]);
		}
		datasource.profile = profile
		return datasource;
	}

	public Object each(Object builder, Object closure) {
		def sql = Sql.newInstance(datasource.url, datasource.username, datasource.password, "com.mysql.jdbc.Driver")
		def summaryData = [:];
		//not sure where to read fiscalYear param from
		def fiscalYear = builder.params.fiscalYear?builder.params.fiscalYear:null;
		def serviceKey = BorrowDirectService.BD_SERVICE_KEY;
		BorrowDirectService.loadDataPerLibrary(sql, fiscalYear, true, summaryData, serviceKey);
		def libraries = BorrowDirectService.getLibraryList(sql, serviceKey)

		def dataRows = []

		//Add row for all
		def currentDataMap = summaryData.get(-1l).borrowing
		def row = getRow("All Libraries", currentDataMap);
		dataRows.add(row)

		libraries.each { library ->
			currentDataMap = summaryData.get(library.library_id.longValue()) != null ? summaryData.get(library.library_id.longValue()).borrowing: [:]
			row = getRow(library.institution, currentDataMap);
			dataRows.add(row)
		}
		return dataRows.each(closure);
	}

	private getRow(libName, currentDataMap){
		def row = [:]
		row.library = libName
		row.req_rec = currentDataMap.turnaroundReqRec!= null?currentDataMap.turnaroundReqRec:0
		row.req_shp = currentDataMap.turnaroundReqShp!= null?currentDataMap.turnaroundReqShp:0
		row.shp_rec = currentDataMap.turnaroundShpRec!= null?currentDataMap.turnaroundShpRec:0
		row.year_items = currentDataMap.currentFiscalYear != null && currentDataMap.currentFiscalYear[-1]!= null?currentDataMap.currentFiscalYear[-1]:0
		row.year_fill_rate = currentDataMap.yearFillRate
		row.year_last_year = currentDataMap.lastFiscalYear != null && currentDataMap.lastFiscalYear[-1] != null?currentDataMap.lastFiscalYear[-1]:0
		(1 .. 12).each { month ->
			row.put(month+'_items', currentDataMap.currentFiscalYear != null && currentDataMap.currentFiscalYear[month]!=null ? currentDataMap.currentFiscalYear[month]:0 )
			row.put(month+'_last_year', currentDataMap.lastFiscalYear != null && currentDataMap.lastFiscalYear[month] != null ? currentDataMap.lastFiscalYear[month]:0 )
		}
		return row;
	}

	/*public static void main(String[] args){
		Datasource ds = new BdSummaryDatasource()
		ds.datasource.url = "";
		ds.datasource.username = "";
		ds.datasource.password = "";
		println ds.getInputs()
		def meta = ds.profile()
		File file = new File("result.csv")
		file.write("");
		def columns = meta.profile.main_report.columns
		columns.each {col ->
			file.append(col.column_name + ", ");
		}
		file.append("\n")
		Closure c = {row ->
			row.each{key, val ->
				file.append(val + ", ");
			}
			file.append("\n");
		}
		ds.each([params:[:]], c);
		println "DONE"
	}*/
}
